% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/redcap-log-read.R
\name{redcap_log_read}
\alias{redcap_log_read}
\title{Get the logging of a project.}
\usage{
redcap_log_read(
  redcap_uri,
  token,
  log_begin_date = Sys.Date() - 7L,
  log_end_date = Sys.Date(),
  record = NULL,
  user = NULL,
  http_response_encoding = "UTF-8",
  locale = readr::default_locale(),
  verbose = TRUE,
  config_options = NULL,
  handle_httr = NULL
)
}
\arguments{
\item{redcap_uri}{The
\href{https://en.wikipedia.org/wiki/Uniform_Resource_Identifier}{uri}/url
of the REDCap server
typically formatted as "https://server.org/apps/redcap/api/".
Required.}

\item{token}{The user-specific string that serves as the password for a
project.  Required.}

\item{log_begin_date}{Return the events occurring after midnight of this
date.
Defaults to the past week; this default mimics the behavior in the browser
and also reduces the strain on your server.}

\item{log_end_date}{Return the events occurring before 24:00 of this date.
Defaults to today.}

\item{record}{Return the events belonging only to specific record
(referring to an existing record name).
Defaults to \code{NULL} which returns
logging activity related to all records.
If a record value passed, it must be a single value.}

\item{user}{Return the events belonging only to specific user
(referring to an existing username).
Defaults to \code{NULL} which returns
logging activity related to all users.
If a user value passed, it must be a single value.}

\item{http_response_encoding}{The encoding value passed to
\code{\link[httr:content]{httr::content()}}.  Defaults to 'UTF-8'.}

\item{locale}{a \code{\link[readr:locale]{readr::locale()}} object to specify preferences like
number, date, and time formats.  This object is passed to
\code{\link[readr:read_delim]{readr::read_csv()}}.  Defaults to \code{\link[readr:locale]{readr::default_locale()}}.}

\item{verbose}{A boolean value indicating if \code{message}s should be printed
to the R console during the operation.  The verbose output might contain
sensitive information (\emph{e.g.} PHI), so turn this off if the output might
be visible somewhere public. Optional.}

\item{config_options}{A list of options passed to \code{\link[httr:POST]{httr::POST()}}.
See details at \code{\link[httr:httr_options]{httr::httr_options()}}. Optional.}

\item{handle_httr}{The value passed to the \code{handle} parameter of
\code{\link[httr:POST]{httr::POST()}}.
This is useful for only unconventional authentication approaches.  It
should be \code{NULL} for most institutions.  Optional.}
}
\value{
Currently, a list is returned with the following elements:
\itemize{
\item \code{data}: An R \code{\link[tibble:tibble]{tibble::tibble()}} of all data access groups of the project.
\item \code{success}: A boolean value indicating if the operation was apparently
successful.
\item \code{status_codes}: A collection of
\href{https://en.wikipedia.org/wiki/List_of_HTTP_status_codes}{http status codes},
separated by semicolons.  There is one code for each batch attempted.
\item \code{outcome_messages}: A collection of human readable strings indicating the
operations' semicolons.  There is one code for each batch attempted.  In an
unsuccessful operation, it should contain diagnostic information.
\item \code{elapsed_seconds}: The duration of the function.
}
}
\description{
This function reads the available logging messages from
REDCap as a \code{\link[tibble:tibble]{tibble::tibble()}}.
}
\examples{
\dontrun{
uri          <- "https://bbmc.ouhsc.edu/redcap/api/"
token        <- "9A81268476645C4E5F03428B8AC3AA7B"

ds_last_week <- REDCapR::redcap_log_read(redcap_uri=uri, token=token)$data
head(ds_last_week)

ds_one_day <-
  REDCapR::redcap_log_read(
    redcap_uri     = uri,
    token          = token,
    log_begin_date = as.Date("2020-08-10"),
    log_end_date   = as.Date("2020-08-10")
  )$data
head(ds_one_day)

ds_one_day_single_record_single_user <-
  REDCapR::redcap_log_read(
    redcap_uri     = uri,
    token          = token,
    log_begin_date = as.Date("2021-07-11"),
    log_end_date   = as.Date("2021-07-11"),
    record         = as.character(3),
    user           = "unittestphifree"
  )$data
head(ds_one_day_single_record_single_user)
}
}
\references{
The official documentation can be found on the 'API Help Page'
and 'API Examples' pages on the REDCap wiki (\emph{i.e.},
https://community.projectredcap.org/articles/456/api-documentation.html
and
https://community.projectredcap.org/articles/462/api-examples.html).
If you do not have an account for the wiki, please ask your campus REDCap
administrator to send you the static material.
}
\author{
Jonathan M. Mang, Will Beasley
}
